@@ -1,11 +1,15 @@
 #define _XOPEN_SOURCE 1
+#define _BSD_SOURCE 1
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <signal.h>
+#include <sys/types.h>
 #include <pcap.h>
+#include <inttypes.h>
+#include <netinet/if_ether.h>
 #if (__GLIBC__)
 #include <net/ethernet.h>
 #endif
@@ -23,33 +27,54 @@
 static unsigned char *device;
 static char errbuf[256];
 
-void error(const char *message) {
+static void error(const char *message) {
   fprintf(stderr,"Error: %s\n",message);
   exit(23);
 }
 
-void error2(const char *message) {
+static void error2(const char *message) {
   fprintf(stderr,"Error: %s: %s\n",message,errbuf);
   exit(23);
 }
 
+static char *ip2a(unsigned int ip) {
+  char *buf=malloc(20);
+  sprintf(buf,"%d.%d.%d.%d",ip>>24,(ip>>16)&0xff,(ip>>8)&0xff,ip&0xff);
+  return buf;
+}
+
+#ifndef OLD_LIBNET
+static libnet_t *l;
+#endif
 
 void sendarp(u_long myip,u_long hisip) {
   static int initialized=0;
-  static struct ether_addr *my_ether;
+#ifdef OLD_LIBNET
   static struct libnet_link_int *l;
-  static u_char enet_bcast[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
-  static u_char enet_nulls[6] = {0,0,0,0,0,0};
+  static struct ether_addr *my_ether;
+#else
+  static struct libnet_ether_addr *my_ether;
+#endif
+  static unsigned char enet_bcast[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
+  static unsigned char enet_nulls[6] = {0,0,0,0,0,0};
   static char *buf;
+
+//  printf("sendarp who-has %s tell %s\n",ip2a(hisip),ip2a(myip));
+
   if (!initialized) {
     buf=malloc(1024);
     if (!buf) error("malloc failed");
+#ifdef OLD_LIBNET
     l=libnet_open_link_interface(device, errbuf);
     if (!l) error2("open_link_interface failed");
     my_ether=libnet_get_hwaddr(l,device,buf);
     if (!my_ether) error2("libnet_get_hwaddr failed");
+#else
+    my_ether=libnet_get_hwaddr(l);
+#endif
     initialized=1;
   }
+#ifdef OLD_LIBNET
   /*
     *  Ethernet header
     */
@@ -80,6 +105,31 @@ void sendarp(u_long myip,u_long hisip) {
 
   if (libnet_write_link_layer(l, device, buf, LIBNET_ARP_H + LIBNET_ETH_H)==-1)
     error2("libnet_write_link_layer failed");
+#else
+  if (libnet_build_arp(
+	  ARPHRD_ETHER,                           /* hardware addr */
+	  ETHERTYPE_IP,                           /* protocol addr */
+	  6,                                      /* hardware addr size */
+	  4,                                      /* protocol addr size */
+	  ARPOP_REQUEST,                          /* operation type */
+	  my_ether->ether_addr_octet,             /* sender hardware addr */
+	  (u_char *)&myip,                        /* sender protocol addr */
+	  enet_nulls,                             /* target hardware addr */
+	  (u_char *)&hisip,                       /* target protocol addr */
+	  NULL,                                   /* payload */
+	  0,                                      /* payload size */
+	  l,                                      /* libnet handle */
+	  0)==-1)                                 /* libnet id */
+             error2("libnet_build_arp failed");
+  if (libnet_autobuild_ethernet(
+	  enet_bcast,                             /* ethernet destination */
+	  ETHERTYPE_ARP,                          /* protocol type */
+	  l)==-1)                                 /* libnet handle */
+             error2("libnet_autobuild_ethernet failed");
+  if (libnet_write(l)==-1)
+    error2("libnet_write failed");
+  libnet_clear_packet(l);
+#endif
 }
 
 struct entry {
@@ -121,12 +171,6 @@ unsigned int parseip(const char *s) {
   return ip;
 }
 
-char *ip2a(unsigned int ip) {
-  char *buf=malloc(20);
-  sprintf(buf,"%d.%d.%d.%d",ip>>24,(ip>>16)&0xff,(ip>>8)&0xff,ip&0xff);
-  return buf;
-}
-
 void parse(char *ptr,struct entry *e) {
   char *colon=strchr(ptr,':');
   char *tmp=strchr(ptr,'/');
@@ -140,11 +184,9 @@ void parse(char *ptr,struct entry *e) {
     e->netmask=0xffffffff ^ ((1 << (32-bits))-1);
     ptr=colon+1;
   }
-/*  printf("/sbin/ifconfig %s ",interface);
-  printip(e->myip);
-  printf(" netmask ");
-  printip(e->netmask);
-  printf("\n"); */
+
+/*  printf("/sbin/ifconfig eth0 %s netmask %s\n",ip2a(e->myip),ip2a(e->netmask)); */
+
   colon=strchr(ptr,':');
   tmp=strchr(ptr,'|');
   if (colon==0) goto parseerror;
@@ -224,6 +266,7 @@ void *send_arps(void *foo) {
     while (tmp) {
       int i;
       for (i=0; i<tmp->pingips; i++)
+//	sendarp(htonl(tmp->myip),htonl(tmp->pingip[i]));
 	sendarp(0,htonl(tmp->pingip[i]));
       tmp=tmp->next;
     }
@@ -254,6 +297,7 @@ int main(int argc,char *argv[]) {
     }
   }
   if (argc>1) device=argv[1];
+#ifdef OLD_LIBNET
   if (device==0) {
     struct sockaddr_in sin;
     if (libnet_select_device(&sin,&device,errbuf) == -1) {
@@ -261,6 +305,11 @@ int main(int argc,char *argv[]) {
       return -1;
     }
   }
+#else
+   l=libnet_init(LIBNET_LINK_ADV,device,errbuf);
+   if (!l) error2("libnet_init failed");
+   device=l->device;
+#endif
   if (getuid()!=0) {
     fprintf(stderr,"Sorry, root only\n");
     exit(1);
@@ -293,20 +342,24 @@ int main(int argc,char *argv[]) {
       fprintf(stderr,"Thread creation failed.\n");
       exit(1);
     }
-    pd = pcap_open_live(device,LIBNET_ARP_H+LIBNET_ETH_H,0,500,errbuf);
+    pd = pcap_open_live(device,50,0,500,errbuf);
     if (pd == NULL)
       error2("pcap_open_live failed");
     for (;(packet = ((u_char *)pcap_next(pd, &pc_hdr)));)
     {
       struct libnet_ethernet_hdr *p;
-      struct libnet_arp_hdr *a;
+      struct ether_arp *a;
+//      printf("packet!\n");
       p = (struct libnet_ethernet_hdr *)(packet);
       if (ntohs(p->ether_type) == ETHERTYPE_ARP) {
-	a = (struct libnet_arp_hdr *)(packet + ETH_H);
-	if (ntohs(a->ar_op) == ARPOP_REPLY) {
+//	printf("arp packet!\n");
+	a = (struct ether_arp *)(packet + 14);
+	if (ntohs(a->arp_op) == ARPOP_REPLY) {
 	  u_long source,target;
-	  source=htonl(*((int*)&(a->ar_spa)));
-	  target=htonl(*((int*)&(a->ar_tpa)));
+//	  printf("arp reply packet!1!!\n");
+	  printf("hardware address length %d, protocol address length %d\n",a->arp_hln,a->arp_pln);
+	  source=htonl(*(int*)(packet+14+8+a->arp_hln));
+	  target=htonl(*(int*)(packet+14+8+a->arp_hln*2+a->arp_pln));
 	  if (verbose)
 	    printf("Got ARP reply from %s to %s\n",ip2a(source),ip2a(target));
 	  {
@@ -374,11 +427,12 @@ int main(int argc,char *argv[]) {
 		  wait(0);
 		  unlink("/etc/resolv.conf");
 		  symlink(tmp->resolvconf,"/etc/resolv.conf");
-		  {
+		  if (tmp->proxyport) {
 		    FILE *f=fopen("/etc/proxy","w");
 		    fprintf(f,"%s:%d\n",tmp->proxy,tmp->proxyport);
 		    fclose(f);
-		  }
+		  } else
+		    unlink("/etc/proxy");
 		  if (tmp->script) {
 		    if (verbose) {
 		      printf("%s \"%s\"\n",tmp->script,tmp->description);
